<?php

if (!defined('CD_PLUGIN_PATH')) {
    define('CD_PLUGIN_PATH', realpath(dirname(__FILE__) . '/../'));
}
if (!defined('CD_FUNC_PATH')) {
    define('CD_FUNC_PATH', CD_PLUGIN_PATH . '/chinadivision');
}

require_once CD_PLUGIN_PATH . '/../../../wp-load.php';
require_once CD_FUNC_PATH . '/chinadivision-config.php';
require_once CD_FUNC_PATH . '/chinadivision-host.php';
require_once CD_FUNC_PATH . '/chinadivision-wc-rest-api.php';

/**
 * @param mixed $data
 * @param int $code
 * @param string $msg
 * @return void
 */
function cd_api_response($data = [], $code = 0, $msg = '')
{
    die(json_encode([
        'code' => $code,
        'data' => $data,
        'msg' => $msg
    ], JSON_UNESCAPED_UNICODE));
}

/**
 * @param mixed $data
 * @param string $msg
 * @return void
 */
function cd_api_response_success($data = [], $msg = 'success')
{
    cd_api_response($data, 0, $msg);
}

/**
 * @param string $msg
 * @param int $code
 * @param mixed $data
 * @return void
 */
function cd_api_response_failed($msg = 'failed', $code = 1, $data = [])
{
    cd_api_response($data, $code, $msg);
}

function cd_str_random($length)
{
    $str = '';
    $chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    for ($i = 0; $i < $length; $i++) {
        $str .= mb_substr($chars, floor(mt_rand(0, strlen($chars) - 1)), 1);
    }
    return $str;
}

function cd_get_domain()
{
    $scheme = $_SERVER['SERVER_PORT'] == '443' ? 'https://' : 'http://';
    return $scheme . $_SERVER['HTTP_HOST'];
}

function cd_get_webhooks($wpdb, $prefix)
{
    $webhooks = $wpdb->get_results("SELECT webhook_id,status,name,delivery_url,topic,secret FROM {$prefix}wc_webhooks WHERE topic IN('order.created', 'order.updated') AND user_id = 1", ARRAY_A);
    $registerWebhook = [];
    foreach ($webhooks as $webhook) {
        if (stripos($webhook['delivery_url'], 'chinadivision-woocommerce') !== false) {
            $registerWebhook[$webhook['topic']] = $webhook;
        }
    }

    return $registerWebhook;
}

function cd_update_shipping($v)
{
    $info = json_decode($v['info'], true);
    $orderId = $v['order_id'] ?? '';
    if (empty($info)) {
        return $orderId;
    }
    if (!cd_check_prefix_is_empty()) {
        $orderId = substr($orderId, strlen(CHINADIVISION_PREFIX));
    }
    if (empty($orderId)) {
        return $orderId;
    }
    $order = (new CD_WC_Rest_Api())->getOrders($orderId);
    if (empty($order)) {
        cd_trace_log('cd_update_shipping failed, order not exist, ' . $orderId);
        return $orderId;
    }
    $existOrderSku = array_column($order['line_items'], 'sku');
    $syncSku = [];
    foreach ($info as $iv) {
        if (!empty($iv['sku']) && in_array($iv['sku'], $existOrderSku)) {
            $syncSku[] = $iv['sku'];
        }
    }
    if (empty($syncSku)) {
        cd_trace_log('cd_update_shipping failed, shipping order sku not exist, ' . $orderId . ', request:' . var_export($info, true));
        return $v['order_id'];
    }

    $updateOrder = [];
    if (!empty($v['shipping_method'])) {
        $shippingId = $order['shipping_lines'][0]['id'] ?? 0;
        if (empty($shippingId)) {
            $updateOrder['shipping_lines'] = [
                [
                    'method_id' => $v['shipping_method'],
                    'method_title' => $v['shipping_method'],
                ]
            ];
        } else {
            $updateOrder['shipping_lines'] = [
                [
                    'id' => $shippingId,
                    'method_id' => $v['shipping_method'],
                    'method_title' => $v['shipping_method'],
                ]
            ];
        }
    }
    if (!empty($v['tracking_number'])) {
        $updateOrder['shipping_lines'][0]['meta_data'] = [
            [
                'key' => 'tracking_number',
                'value' => $v['tracking_number'],
            ]
        ];
    }
    if (!empty($updateOrder)) {
        $ret = (new CD_WC_Rest_Api())->updateOrder($orderId, $updateOrder);
        if (isset($ret['code'])) {
            cd_trace_log('cd_update_shipping failed, updateOrder failed: ' . var_export($ret, true));
            return $orderId;
        }
    }
    return true;
}

function cd_http($url, $method = 'POST', $data = [], $header = [])
{
    cd_trace_log('send api params');
    cd_trace_log(func_get_args());

    $method = strtoupper($method);
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
    if ($method == 'POST') {
        curl_setopt($ch, CURLOPT_POSTFIELDS, is_array($data) ? json_encode($data) : $data);
        curl_setopt($ch, CURLOPT_POST, true);
    } elseif ($method == 'PUT' || $method == 'DELETE') {
        curl_setopt($ch, CURLOPT_POSTFIELDS, is_array($data) ? json_encode($data) : $data);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
    }
    if (stripos($url, 'https://') === 0) {
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
    }
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_URL, $url);
    if (curl_errno($ch)) {
        $error = curl_error($ch);
    }
    $res = curl_exec($ch);
    curl_close($ch);
    if (isset($error)) {
        return ['code' => -9999, 'msg' => $error];
    }

    cd_trace_log('api response');
    cd_trace_log($res);

    $ret = json_decode($res, true);
    if (is_array($ret)) {
        return $ret;
    }
    return $res;
}

function cd_create_order($order_id)
{
    $order_id = (int)$order_id;
    if (empty($order_id)) {
        return false;
    }
    $order = (new CD_WC_Rest_Api())->getOrders($order_id);
    if (empty($order) || isset($order['code'])) {
        cd_trace_log('cd_create_order(' .$order_id. ') get order failed, ' . var_export($order, true));
        return false;
    }
    $quantity = 0;
    $shipping_method = '';
    $info = [];

    foreach ($order['line_items'] as $item) {
        $quantity += $item['quantity'];
        $info[] = [
            'product_id' => $item['product_id'],
            'quantity' => $item['quantity'],
            'product_name' => $item['name'],
            'sku' => $item['sku'],
            'product_url' => $item['image']['src'] ?? '',
            'color' => $item['color'] ?? '',
        ];
    }
    foreach ($order['shipping_lines'] as $item) {
        $shipping_method = $item['method_title'];
    }

    $post_data = [];
    $post_data['order_id'] = cd_check_prefix_is_empty() ? $order['id'] : (CHINADIVISION_PREFIX . $order['id']);
    $post_data['first_name'] = $order['shipping']['first_name'];
    $post_data['last_name'] = $order['shipping']['last_name'];
    $post_data['ship_address1'] = $order['shipping']['address_1'];

    $post_data['ship_address2'] = $order['shipping']['address_2'];
    $post_data['ship_city'] = $order['shipping']['city'];
    $post_data['ship_state'] = $order['shipping']['state'];
    $post_data['ship_zip'] = $order['shipping']['postcode'];

    $post_data['ship_country'] = $order['shipping']['country'];
    $post_data['ship_phone'] = $order['billing']['phone'];
    $post_data['ship_email'] = $order['billing']['email'];
    $post_data['quantity'] = $quantity;
    $post_data['shipping_method'] = $shipping_method;

    $post_data['inspection'] = 1;
    $post_data['remark'] = $order['customer_note'];
    $post_data['business_type'] = 1;
    $post_data['info'] = $info;
    $post_data['plug_in_name'] = 'WOOCOMMERCE';

    $header = [
        'Content-Type:application/x-www-form-urlencoded',
        'apikey:' . CHINADIVISION_API_KEY,
    ];

    return cd_http(CD_CREATE_ORDER_URL, 'POST', $post_data, $header);
}

function cd_check_apikey_is_empty()
{
    if (CHINADIVISION_API_KEY == "your apikey" || CHINADIVISION_API_KEY == "") {
        return true;
    }
    return false;
}

function cd_check_prefix_is_empty()
{
    if (CHINADIVISION_API_KEY == "your prefix" || CHINADIVISION_API_KEY == "") {
        return true;
    }
    return false;
}

function cd_trace_log($data, $isSaveHeader = false)
{
    if (defined('CD_DEBUG_INFO') && !CD_DEBUG_INFO) {
        return;
    }
    $file = cd_get_plugin_main_path() . '/chinadivision-trace.log';
    $now = date('c');
    $headerStr = '';
    if ($isSaveHeader) {
        $headerStr = "\nheader: " . json_encode($_SERVER, JSON_UNESCAPED_UNICODE) . "\n";
    }
    $content = "---- now content [$now] ----" . $headerStr . "---- Log ----\n" . var_export($data, true) . "\n";
    file_put_contents($file, $content, FILE_APPEND);
}

function cd_edit_constant($filename, $constantName, $constantValue, $comment = '')
{
    $content = file_get_contents($filename);

    $constantTemplate = "define('%s', %s);";

    $constantPattern = "/define\('$constantName',\s*([^)]+)\);/";
    $matches = [];
    preg_match($constantPattern, $content, $matches);

    if (is_string($constantValue)) {
        $constantValue = "'$constantValue'";
    }

    if (count($matches) > 0) {
        $replacement = sprintf($constantTemplate, $constantName, $constantValue);
        $content = preg_replace($constantPattern, $replacement, $content);
    } else {
        if (!empty($comment)) {
            $content .= PHP_EOL .'/** '. $comment . ' */';
        }
        $content .= PHP_EOL . sprintf($constantTemplate, $constantName, $constantValue);
    }

    file_put_contents($filename, $content);
}

function cd_get_plugin_main_path()
{
    return CD_PLUGIN_PATH;
}